/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2002 USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): .

====================================================================*/

package org.objectweb.corba.generator.idl.lib;


/** To access all the AST Declaration. */
import org.objectweb.corba.ast.api.Declaration;

/** To access AST DeclarationKind. */
import org.objectweb.corba.ast.api.DeclarationKind;

/** To access all the AST Scope. */
import org.objectweb.corba.ast.api.Scope;

/** To access Generation Exception. */
import org.objectweb.corba.generator.common.lib.GenerationException;

/** To use ArrayList. */
import java.util.ArrayList;

/** To access Vector. */
import java.util.Vector;

/** To use Hashtable. */
import java.util.Hashtable;


public class IDL2Generator
     extends IDL3Generator
  implements org.objectweb.corba.generator.idl.api.IDL2Generator
{

    // ===========================================================
    //
    // Internal State.
    //
    // ===========================================================

    /**
     * All local declarations.
     * key = parent_id
     * value = an array list of childs
     **/
    protected Hashtable local_decls_;

    /**
     * True if declarations are mapped into a single file.
     **/
    protected boolean single_file_;


    // ===========================================================
    //
    // Constructors.
    //
    // ===========================================================

    /**
     * The default constructor.
     *
     * @param ast - The Abstract Syntax Tree.
     */

    public IDL2Generator(org.objectweb.corba.ast.api.AST ast)
    {
        // Call the IDL3Generator constructor
        super(ast);

        single_file_ = false;
    }

    // ===========================================================
    //
    // Internal methods.
    //
    // ===========================================================

    /**
     * Put a declaration in the Hashtable.
     **/
    private void
    putDecl(Declaration child)
    {
        Declaration parent = child.getParent();
        String parent_id = parent.getId();
        ArrayList childs = null;

        // Is the parent in the Hashtable ?
        childs = (ArrayList)local_decls_.get(parent_id);
        if (childs == null)
        {
            childs = new ArrayList();
            childs.add(child);
            local_decls_.put(parent_id, childs);
            // Add the parent if necessary
            if (parent.getId() != target_decl_.getId())
                putDecl(parent);
        }
        else
        {
            // Only add the declaration to childs
            if (!childs.contains(child))
                childs.add(child);
        }
    }

    /**
     * Generate local idl2.
     *
     * @param id - The file identifier for local declarations generation.
     **/
    private void
    generate_local_idl2(String id)
    {
        ArrayList decls = (ArrayList)local_decls_.get(id);
        java.util.Iterator it = decls.iterator();

        while (it.hasNext())
        {
            Declaration decl = (Declaration)it.next();
            long dk = decl.getDeclKind();

            if ( dk == DeclarationKind.dk_module)
            {
                print(macro(decl));
                print("module " + decl.getName() + "\n");
                print("{/inc\n");
                local_forward( (org.objectweb.corba.ast.api.Scope)decl );
                map("END_LINE");
                generate_local_idl2(decl.getId());
                print("/dec};\n");
            }
            else
            {
                // Map local interfaces
                String decl_id = org.objectweb.corba.ast.lib.DeclarationKindImpl.toString(dk).toUpperCase();
                put("obj", decl);
                map(decl_id);
            }
        }
    }

    /**
     * Method called after generation.
     * Generally used to close files.
     **/
    protected void
    after_generation()
    {
        super.after_generation();

        // Generate local idl2
        if ( !single_file_ )
        {
            switchToFile("local");
            last_prefix_ = "";
            generate_local_idl2("root");
            close("local");
        }
    }

    // ===========================================================
    //
    // Useful methods for IDL2 Mapping.
    //
    // ===========================================================

    /**
     * Utility method to check forward declarations.
     *
     * @return The associated search mask.
     **/
    public long
    forward_check()
    {
        //
        //  TODO : do not forward struct and unions because no orb supports it yet !
        //
        return ( DeclarationKind.dk_interface +
                 DeclarationKind.dk_local_interface +
                 DeclarationKind.dk_abstract_interface +
                 DeclarationKind.dk_component +
                 DeclarationKind.dk_value +
                 //                DeclarationKind.dk_struct +
                 //                DeclarationKind.dk_union +
                 DeclarationKind.dk_event );
    }

    // ===========================================================
    //
    // Methods for IDL2 Mapping.
    //
    // ===========================================================


    // ===========================================================
    //
    // Public methods.
    //
    // ===========================================================

    /**
     * Initialize the generator.
     * If outputfile equals to outputfile_local, declarations will be
     * genarated in the same file.
     *
     * @param outputfile - The name where declarations will be generated.
     * @param outputfile_loacal - The file name to write local declarations in.
     * @param includes - files to include (with <file>).
     * @param userIncludes - user files to include (with "file").
     * @param app_name - The application's name.
     **/
    public void
    initialize( String outputfile,
                String outputfile_local,
                ArrayList includes,
                ArrayList userIncludes,
                String app_name )
    {
        super.initialize( outputfile,
                          new ArrayList(),
                          app_name );

        /** Compare input and output file **/
        single_file_ = outputfile.equals(outputfile_local);

        /** Initialize velocity context **/
        put("userIncludes", userIncludes);
        put("includes", includes);

        if ( !single_file_ )
        {
            /** The output file for local declarations **/
            open(outputfile_local, "local");
            map("LICENSE", "local");
            // Import non local IDL2 declarations
            java.io.File out = new java.io.File(outputfile);
            print("#include \"" + out.getName() + "\"\n\n", "local");

            /** Set the default output **/
            switchToFile("out");
        }
    }

    /**
     * Generates contents of a scope.
     *
     * @param target_decl - The scope to visit.
     *
     * @throws GenerationException
     **/
    public void
    generate(Scope target_decl)
    throws GenerationException
    {
        // Init local declarations hashtable
        local_decls_ = new Hashtable();
        ArrayList childs = new ArrayList();
        childs.add(target_decl);
        local_decls_.put("root", childs);

        super.generate(target_decl);
    }

    /**
     * Generates a vector of declarations.
     *
     * @param decls - Declarations to visit.
     **/
    public void
    generate(Vector decls)
    throws org.objectweb.corba.generator.common.lib.GenerationException
    {
        put("obj", decls);
        map("FILE_HEADER");
        visit(decls);
        after_generation();
    }

    /**
     * Visit a single contained declaration
     *
     * @param contained - The declaration to visit.
     * @param extension - used to map id
     **/
    public void
    visitContained(Declaration contained,
                   String extension)
    {
        if ( single_file_ )
        {
            super.visitContained(contained, extension);
        }
        else
        {
            long dk = contained.getDeclKind();
            String id = org.objectweb.corba.ast.lib.DeclarationKindImpl.toString(dk).toUpperCase()+ extension;

            put("obj", contained);
            // System.err.println("Visit contained :"+contained.getAbsoluteName()+" | "+id+" | "+contained.getPrefix() );
            if ( dk == DeclarationKind.dk_local_interface)
            {
                putDecl(contained);
            }
            else
            {
                map(id);
            }
        }
    }

    /**
     * Generate forwards declaration from a scope for local declarations.
     *
     * @param scope - The Scope containing declarations.
     **/
    public void
    local_forward(org.objectweb.corba.ast.api.Scope scope)
    {
        Declaration[] objs = null;
        objs = scope.getContents(true, DeclarationKind.dk_local_interface);

        for (int i=0;i<objs.length;i++)
            super.visitContained(objs[i], "_FORWARD");

        put("obj", scope);
    }

}
