/*====================================================================

ObjectWeb Util Preprocessor Package.
Copyright (C) 2003 INRIA & USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.util.cpp.lib;

// Package dependencies.
import org.objectweb.util.cmdline.lib.ApplicationBase;
import org.objectweb.util.cmdline.api.CommandLine;
import org.objectweb.util.cmdline.lib.DefaultCommandLine;
import org.objectweb.util.cpp.api.Preprocessor;

/**
 * This provides a preprocessor written in Java.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *
 * @version 0.1
 */

public class PreprocessorApplication
     extends ApplicationBase
  implements org.objectweb.util.cpp.api.PreprocessorApplication
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The preprocessor. */
    private Preprocessor preprocessor_;

    /** Option -P */
    private OptionP optionP_;

    /** Option -DNAME */
    private OptionDNAME optionDNAME_;

    /** Option -UNAME */
    private OptionUNAME optionUNAME_;

    /** Option -IDIR */
    private OptionIDIR optionIDIR_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /** The default constructor. */
    public
    PreprocessorApplication()
    {
        this(new DefaultCommandLine("preprocessor",
                                    "file",
                                    "Preprocess a file.",
				    true),
             true);
    }

    /**
     * The constructor with a provided command line manager.
     *
     * @param commandLine The provided command line manager.
     * @param withDefaultOptions True to add default options in the command line.
     */
    public
    PreprocessorApplication(CommandLine commandLine,
                            boolean withDefaultOptions)
    {
        // Calls the default ApplicationBase constructor.
        super(commandLine, withDefaultOptions);

        // Inits internal state.
        setPreprocessor(new PreprocessorJPP());

        // Creates the preprocessor options.
        commandLine.addOption(new OptionCPP(this));

        optionP_ = new OptionP(getPreprocessor());
        commandLine.addOption(optionP_);
        optionDNAME_ = new OptionDNAME(getPreprocessor());
        commandLine.addOption(optionDNAME_);
        optionUNAME_ = new OptionUNAME(getPreprocessor());
        commandLine.addOption(optionUNAME_);
        optionIDIR_ = new OptionIDIR(getPreprocessor());
        commandLine.addOption(optionIDIR_);
    }

    // ==================================================================
    //
    //  Public methods for org.objectweb.util.api.Identifiable
    //
    // ==================================================================

    // ==================================================================
    //
    //  Public methods for org.objectweb.util.cmdline.api.Application
    //
    // ==================================================================

    /**
     * Starts the main function.
     *
     * Must be defined into subclasses.
     *
     * @param args The command line arguments.
     *
     * @return The status of the application.
     */
    public int
    start(String[] args)
    {
       boolean result = preprocess(args[0], getConsole().getOutputStream());
       return result?0:-1;
    }

    // ==================================================================
    //
    //  Public methods for org.objectweb.util.cpp.api.PreprocessorHolder
    //
    // ==================================================================

    /**
     * Obtains the associated preprocessor.
     *
     * @return The associated preprocessor.
     */
    public Preprocessor
    getPreprocessor()
    {
        return preprocessor_;
    }

    /**
     * Sets the associated preprocessor.
     *
     * @param preprocessor The associated preprocessor.
     */
    public void
    setPreprocessor(Preprocessor preprocessor)
    {
        preprocessor_ = preprocessor;
        preprocessor_.setConsole(getConsole());
        if(optionP_ != null)
            optionP_.setPreprocessor(preprocessor);
        if(optionDNAME_ != null)
            optionDNAME_.setPreprocessor(preprocessor);
        if(optionUNAME_ != null)
            optionUNAME_.setPreprocessor(preprocessor);
        if(optionIDIR_ != null)
            optionIDIR_.setPreprocessor(preprocessor);
    }

    // ==================================================================
    //
    //  Public methods for org.objectweb.util.cpp.api.PreprocessorOperations
    //
    // ==================================================================

    /**
     * Preprocesses a file.
     *
     * @param fileName The name of the file to preprocess.
     *
     * @return The file containing the output file.
     */
    public java.io.File
    preprocess(String fileName)
    {
         getConsole().message("Preprocessing file " + fileName + "...");

         java.io.File result = getPreprocessor().preprocess(fileName);

         // If error
         if (result == null)
         {
             getConsole().error("Preprocessing file " + fileName + " failed!");
             return null;
         }

         getConsole().message("File " + fileName + " preprocessed.");
         return result;
    }

    /**
     * Preprocesses a file.
     *
     * @param fileName The name of the file to preprocess.
     * @param tmpFile The file containing the output file.
     *
     * @return True if OK.
     */
    public boolean
    preprocess(String fileName,
               java.io.File tmpFile)
    {
         getConsole().message("Preprocessing file " + fileName + "...");

         boolean result = getPreprocessor().preprocess(fileName, tmpFile);

         // If error
         if (!result)
         {
             getConsole().error("Preprocessing file " + fileName + " failed!");
             return false;
         }

         getConsole().message("File " + fileName + " preprocessed.");
         return true;
    }

    /**
     * Preprocesses a file.
     *
     * @param fileName The name of the file to preprocess.
     * @param output The output stream.
     */
    public boolean
    preprocess(String fileName,
               java.io.OutputStream output)
    {
         getConsole().message("Preprocessing file " + fileName + "...");

         boolean result = getPreprocessor().preprocess(fileName, output);

         // If error
         if (!result)
         {
             getConsole().error("Preprocessing file " + fileName + " failed!");
             return false;
         }

         getConsole().message("File " + fileName + " preprocessed.");
         return true;
    }

    // ==================================================================
    //
    //  Public methods for org.objectweb.util.cpp.api.PreprocessorApplication
    //
    // ==================================================================

    // ==================================================================
    //
    //  Public static methods.
    //
    // ==================================================================

    /**
     * The main bootstrap method.
     *
     * @param args The command line arguments.
     */
    public static void
    main(String[] args)
    {
        PreprocessorApplication application = new PreprocessorApplication();
        application.runMain(args);
    }
}
